@@ -4,34 +4,34 @@ mod selectors_test;
 
 use crate::construction::heuristics::*;
 use crate::models::problem::Job;
-use crate::utils::{map_reduce, parallel_collect, Either, Noise};
+use crate::models::solution::Leg;
+use crate::utils::*;
 use rand::prelude::*;
+use std::sync::Arc;
 
 /// On each insertion step, selects a list of routes where jobs can be inserted.
 /// It is up to implementation to decide whether list consists of all possible routes or just some subset.
 pub trait RouteSelector {
     /// Returns routes for job insertion.
-    fn select<'a>(&'a self, ctx: &'a mut InsertionContext, jobs: &[Job])
-        -> Box<dyn Iterator<Item = RouteContext> + 'a>;
+    fn select<'a>(
+        &'a self,
+        insertion_ctx: &'a mut InsertionContext,
+        jobs: &[Job],
+    ) -> Box<dyn Iterator<Item = RouteContext> + 'a>;
 }
 
 /// Returns a list of all possible routes for insertion.
+#[derive(Default)]
 pub struct AllRouteSelector {}
 
-impl Default for AllRouteSelector {
-    fn default() -> Self {
-        Self {}
-    }
-}
-
 impl RouteSelector for AllRouteSelector {
     fn select<'a>(
         &'a self,
-        ctx: &'a mut InsertionContext,
+        insertion_ctx: &'a mut InsertionContext,
         _jobs: &[Job],
     ) -> Box<dyn Iterator<Item = RouteContext> + 'a> {
-        ctx.solution.routes.shuffle(&mut ctx.environment.random.get_rng());
-        Box::new(ctx.solution.routes.iter().cloned().chain(ctx.solution.registry.next()))
+        insertion_ctx.solution.routes.shuffle(&mut insertion_ctx.environment.random.get_rng());
+        Box::new(insertion_ctx.solution.routes.iter().cloned().chain(insertion_ctx.solution.registry.next()))
     }
 }
 
@@ -43,19 +43,14 @@ pub trait JobSelector {
 }
 
 /// Returns a list of all jobs to be inserted.
+#[derive(Default)]
 pub struct AllJobSelector {}
 
-impl Default for AllJobSelector {
-    fn default() -> Self {
-        Self {}
-    }
-}
-
 impl JobSelector for AllJobSelector {
-    fn select<'a>(&'a self, ctx: &'a mut InsertionContext) -> Box<dyn Iterator<Item = Job> + 'a> {
-        ctx.solution.required.shuffle(&mut ctx.environment.random.get_rng());
+    fn select<'a>(&'a self, insertion_ctx: &'a mut InsertionContext) -> Box<dyn Iterator<Item = Job> + 'a> {
+        insertion_ctx.solution.required.shuffle(&mut insertion_ctx.environment.random.get_rng());
 
-        Box::new(ctx.solution.required.iter().cloned())
+        Box::new(insertion_ctx.solution.required.iter().cloned())
     }
 }
 
@@ -64,27 +59,30 @@ pub trait InsertionEvaluator {
     /// Evaluates insertion of a single job into given collection of routes.
     fn evaluate_job(
         &self,
-        ctx: &InsertionContext,
+        insertion_ctx: &InsertionContext,
         job: &Job,
         routes: &[RouteContext],
+        leg_selector: &(dyn LegSelector + Send + Sync),
         result_selector: &(dyn ResultSelector + Send + Sync),
     ) -> InsertionResult;
 
     /// Evaluates insertion of multiple jobs into given route.
     fn evaluate_route(
         &self,
-        ctx: &InsertionContext,
-        route: &RouteContext,
+        insertion_ctx: &InsertionContext,
+        route_ctx: &RouteContext,
         jobs: &[Job],
+        leg_selector: &(dyn LegSelector + Send + Sync),
         result_selector: &(dyn ResultSelector + Send + Sync),
     ) -> InsertionResult;
 
     /// Evaluates insertion of a job collection into given collection of routes.
     fn evaluate_all(
         &self,
-        ctx: &InsertionContext,
+        insertion_ctx: &InsertionContext,
         jobs: &[Job],
         routes: &[RouteContext],
+        leg_selector: &(dyn LegSelector + Send + Sync),
         result_selector: &(dyn ResultSelector + Send + Sync),
     ) -> InsertionResult;
 }
@@ -109,15 +107,18 @@ impl PositionInsertionEvaluator {
     /// Evaluates all jobs ad routes.
     pub(crate) fn evaluate_and_collect_all(
         &self,
-        ctx: &InsertionContext,
+        insertion_ctx: &InsertionContext,
         jobs: &[Job],
         routes: &[RouteContext],
+        leg_selector: &(dyn LegSelector + Send + Sync),
         result_selector: &(dyn ResultSelector + Send + Sync),
     ) -> Vec<InsertionResult> {
-        if Self::is_fold_jobs(ctx) {
-            parallel_collect(jobs, |job| self.evaluate_job(ctx, job, routes, result_selector))
+        if Self::is_fold_jobs(insertion_ctx) {
+            parallel_collect(jobs, |job| self.evaluate_job(insertion_ctx, job, routes, leg_selector, result_selector))
         } else {
-            parallel_collect(routes, |route| self.evaluate_route(ctx, route, jobs, result_selector))
+            parallel_collect(routes, |route_ctx| {
+                self.evaluate_route(insertion_ctx, route_ctx, jobs, leg_selector, result_selector)
+            })
         }
     }
 
@@ -131,48 +132,56 @@ impl PositionInsertionEvaluator {
 impl InsertionEvaluator for PositionInsertionEvaluator {
     fn evaluate_job(
         &self,
-        ctx: &InsertionContext,
+        insertion_ctx: &InsertionContext,
         job: &Job,
         routes: &[RouteContext],
+        leg_selector: &(dyn LegSelector + Send + Sync),
         result_selector: &(dyn ResultSelector + Send + Sync),
     ) -> InsertionResult {
+        let eval_ctx =
+            EvaluationContext { constraint: &insertion_ctx.problem.constraint, job, leg_selector, result_selector };
+
         routes.iter().fold(InsertionResult::make_failure(), |acc, route_ctx| {
-            evaluate_job_insertion_in_route(ctx, route_ctx, job, self.insertion_position, acc, result_selector)
+            evaluate_job_insertion_in_route(insertion_ctx, &eval_ctx, route_ctx, self.insertion_position, acc)
         })
     }
 
     fn evaluate_route(
         &self,
-        ctx: &InsertionContext,
-        route: &RouteContext,
+        insertion_ctx: &InsertionContext,
+        route_ctx: &RouteContext,
         jobs: &[Job],
+        leg_selector: &(dyn LegSelector + Send + Sync),
         result_selector: &(dyn ResultSelector + Send + Sync),
     ) -> InsertionResult {
         jobs.iter().fold(InsertionResult::make_failure(), |acc, job| {
-            evaluate_job_insertion_in_route(ctx, route, job, self.insertion_position, acc, result_selector)
+            let eval_ctx =
+                EvaluationContext { constraint: &insertion_ctx.problem.constraint, job, leg_selector, result_selector };
+            evaluate_job_insertion_in_route(insertion_ctx, &eval_ctx, route_ctx, self.insertion_position, acc)
         })
     }
 
     fn evaluate_all(
         &self,
-        ctx: &InsertionContext,
+        insertion_ctx: &InsertionContext,
         jobs: &[Job],
         routes: &[RouteContext],
+        leg_selector: &(dyn LegSelector + Send + Sync),
         result_selector: &(dyn ResultSelector + Send + Sync),
     ) -> InsertionResult {
-        if Self::is_fold_jobs(ctx) {
+        if Self::is_fold_jobs(insertion_ctx) {
             map_reduce(
                 jobs,
-                |job| self.evaluate_job(ctx, job, routes, result_selector),
+                |job| self.evaluate_job(insertion_ctx, job, routes, leg_selector, result_selector),
                 InsertionResult::make_failure,
-                |a, b| result_selector.select_insertion(ctx, a, b),
+                |a, b| result_selector.select_insertion(insertion_ctx, a, b),
             )
         } else {
             map_reduce(
                 routes,
-                |route| self.evaluate_route(ctx, route, jobs, result_selector),
+                |route| self.evaluate_route(insertion_ctx, route, jobs, leg_selector, result_selector),
                 InsertionResult::make_failure,
-                |a, b| result_selector.select_insertion(ctx, a, b),
+                |a, b| result_selector.select_insertion(insertion_ctx, a, b),
             )
         }
     }
@@ -199,14 +208,9 @@ pub trait ResultSelector {
 }
 
 /// Selects best result.
+#[derive(Default)]
 pub struct BestResultSelector {}
 
-impl Default for BestResultSelector {
-    fn default() -> Self {
-        Self {}
-    }
-}
-
 impl ResultSelector for BestResultSelector {
     fn select_insertion(&self, _: &InsertionContext, left: InsertionResult, right: InsertionResult) -> InsertionResult {
         InsertionResult::choose_best_result(left, right)
@@ -255,3 +259,55 @@ impl ResultSelector for NoiseResultSelector {
         }
     }
 }
+
+// TODO avoid boxing in interfaces
+/// Provides the way to select legs from the tour which should be used for insertion analysis.
+pub trait LegSelector {
+    /// Returns legs from the tour of given route context while inserting the given job.
+    fn get_legs<'a>(
+        &self,
+        route_ctx: &'a RouteContext,
+        job: &Job,
+        skip: usize,
+    ) -> Box<dyn Iterator<Item = Leg<'a>> + 'a>;
+}
+
+/// Selects different legs based on route and job size.
+pub struct VariableLegSelector {
+    random: Arc<dyn Random + Send + Sync>,
+}
+
+impl VariableLegSelector {
+    /// Creates a new instance of `VariableLegSelector`.
+    pub fn new(random: Arc<dyn Random + Send + Sync>) -> Self {
+        Self { random }
+    }
+}
+
+impl LegSelector for VariableLegSelector {
+    fn get_legs<'a>(
+        &self,
+        route_ctx: &'a RouteContext,
+        job: &Job,
+        skip: usize,
+    ) -> Box<dyn Iterator<Item = Leg<'a>> + 'a> {
+        let (greedy_threshold, sample_size) = match job {
+            Job::Single(_) => (24, 16),
+            Job::Multi(multi) if multi.jobs.len() == 2 => (16, 8),
+            Job::Multi(_) => (16, 4),
+        };
+
+        let total_legs = route_ctx.route.tour.legs().size_hint().0;
+        let visit_legs = if total_legs > skip { total_legs - skip } else { 0 };
+
+        if visit_legs < greedy_threshold {
+            Box::new(route_ctx.route.tour.legs().skip(skip))
+        } else {
+            Box::new(SelectionSamplingIterator::new(
+                route_ctx.route.tour.legs().skip(skip),
+                sample_size,
+                self.random.clone(),
+            ))
+        }
+    }
+}
