@@ -66,7 +66,7 @@ pub enum TimeSpan {
 }
 
 /// Specifies a flexible time interval.
-#[derive(Clone, Debug)]
+#[derive(Clone, Debug, Default)]
 pub struct TimeInterval {
     /// Earliest possible time to start.
     pub earliest: Option<Timestamp>,
@@ -178,12 +178,6 @@ impl TimeSpan {
     }
 }
 
-impl Default for TimeInterval {
-    fn default() -> Self {
-        Self { earliest: None, latest: None }
-    }
-}
-
 impl TimeInterval {
     /// Converts time interval to time window.
     pub fn to_time_window(&self) -> TimeWindow {
