@@ -103,7 +103,7 @@ impl<T: Load + Add<Output = T> + Sub<Output = T> + 'static> DemandDimension<T> f
 }
 
 /// Specifies single dimensional load type.
-#[derive(Clone, Copy, Debug)]
+#[derive(Clone, Copy, Debug, Default)]
 pub struct SingleDimLoad {
     /// An actual load value.
     pub value: i32,
@@ -116,12 +116,6 @@ impl SingleDimLoad {
     }
 }
 
-impl Default for SingleDimLoad {
-    fn default() -> Self {
-        Self { value: 0 }
-    }
-}
-
 impl Load for SingleDimLoad {
     fn is_not_empty(&self) -> bool {
         self.value != 0
