@@ -85,14 +85,9 @@ pub trait ActivityCost {
 }
 
 /// Default activity costs.
+#[derive(Default)]
 pub struct SimpleActivityCost {}
 
-impl Default for SimpleActivityCost {
-    fn default() -> Self {
-        Self {}
-    }
-}
-
 impl ActivityCost for SimpleActivityCost {}
 
 /// Provides the way to get routing information for specific locations and actor.
