@@ -41,17 +41,7 @@ impl StaticSelective {
     /// Creates an instance of `StaticSelective` with default parameters.
     pub fn new_with_defaults(problem: Arc<Problem>, environment: Arc<Environment>) -> Self {
         let default_mutation = Self::create_default_mutation(problem, environment.clone());
-        let local_search = Arc::new(LocalSearch::new(Arc::new(CompositeLocalOperator::new(
-            vec![
-                (Arc::new(ExchangeInterRouteBest::default()), 100),
-                (Arc::new(ExchangeInterRouteRandom::default()), 30),
-                (Arc::new(ExchangeIntraRouteRandom::default()), 30),
-                (Arc::new(ExchangeSequence::default()), 100),
-                (Arc::new(RescheduleDeparture::default()), 20),
-            ],
-            1,
-            2,
-        ))));
+        let local_search = Self::create_default_local_search(environment.clone());
 
         Self::new(vec![
             (
@@ -100,22 +90,26 @@ impl StaticSelective {
         problem: Arc<Problem>,
         environment: Arc<Environment>,
     ) -> Arc<dyn Mutation + Send + Sync> {
+        let random = environment.random.clone();
         // initialize recreate
         let recreate = Arc::new(WeightedRecreate::new(vec![
-            (Arc::new(RecreateWithSkipBest::new(1, 2)), 50),
-            (Arc::new(RecreateWithRegret::new(2, 3)), 20),
-            (Arc::new(RecreateWithCheapest::default()), 20),
-            (Arc::new(RecreateWithPerturbation::new_with_defaults(environment.random.clone())), 10),
-            (Arc::new(RecreateWithSkipBest::new(3, 4)), 5),
-            (Arc::new(RecreateWithGaps::default()), 5),
+            (Arc::new(RecreateWithSkipBest::new(1, 2, random.clone())), 50),
+            (Arc::new(RecreateWithRegret::new(2, 3, random.clone())), 20),
+            (Arc::new(RecreateWithCheapest::new(random.clone())), 20),
+            (Arc::new(RecreateWithPerturbation::new_with_defaults(random.clone())), 10),
+            (Arc::new(RecreateWithSkipBest::new(3, 4, random.clone())), 5),
+            (Arc::new(RecreateWithGaps::new(2, 20, random.clone())), 5),
             // TODO use dimension size from problem
-            (Arc::new(RecreateWithBlinks::<SingleDimLoad>::new_with_defaults(environment.random.clone())), 5),
-            (Arc::new(RecreateWithFarthest::default()), 2),
-            (Arc::new(RecreateWithSkipBest::new(4, 8)), 2),
-            (Arc::new(RecreateWithNearestNeighbor::default()), 1),
-            (Arc::new(RecreateWithSlice::default()), 1),
+            (Arc::new(RecreateWithBlinks::<SingleDimLoad>::new_with_defaults(random.clone())), 5),
+            (Arc::new(RecreateWithFarthest::new(random.clone())), 2),
+            (Arc::new(RecreateWithSkipBest::new(4, 8, random.clone())), 2),
+            (Arc::new(RecreateWithNearestNeighbor::new(random.clone())), 1),
+            (Arc::new(RecreateWithSlice::new(random.clone())), 1),
             (
-                Arc::new(RecreateWithSkipRandom::default_explorative_phased(Arc::new(RecreateWithCheapest::default()))),
+                Arc::new(RecreateWithSkipRandom::default_explorative_phased(
+                    Arc::new(RecreateWithCheapest::new(random.clone())),
+                    random.clone(),
+                )),
                 1,
             ),
         ]));
@@ -130,12 +124,21 @@ impl StaticSelective {
             (vec![(Arc::new(AdjustedStringRemoval::default()), 1.), (random_ruin.clone(), 0.1)], 100),
             (vec![(Arc::new(NeighbourRemoval::default()), 1.), (random_ruin.clone(), 0.1)], 10),
             (vec![(Arc::new(WorstJobRemoval::default()), 1.), (random_ruin.clone(), 0.1)], 10),
-            (vec![(Arc::new(ClusterRemoval::new_with_defaults(problem, environment)), 1.), (random_ruin, 0.1)], 5),
+            (
+                vec![
+                    (Arc::new(ClusterRemoval::new_with_defaults(problem, environment.clone())), 1.),
+                    (random_ruin, 0.1),
+                ],
+                5,
+            ),
             (vec![(close_route, 1.), (random_job.clone(), 0.1)], 2),
             (vec![(random_route, 1.), (random_job, 0.1)], 1),
         ]));
 
-        Arc::new(RuinAndRecreate::new(ruin, recreate))
+        Arc::new(WeightedMutation::new(
+            vec![Arc::new(RuinAndRecreate::new(ruin, recreate)), Self::create_default_local_search(environment)],
+            vec![100, 10],
+        ))
     }
 
     /// Creates default random ruin method.
@@ -146,6 +149,23 @@ impl StaticSelective {
             (vec![(Arc::new(RandomJobRemoval::new(RuinLimits::default())), 1.)], 2),
         ]))
     }
+
+    fn create_default_local_search(environment: Arc<Environment>) -> Arc<dyn Mutation + Send + Sync> {
+        let random = environment.random.clone();
+
+        Arc::new(LocalSearch::new(Arc::new(CompositeLocalOperator::new(
+            vec![
+                (Arc::new(ExchangeSwapStar::new(random)), 200),
+                (Arc::new(ExchangeInterRouteBest::default()), 100),
+                (Arc::new(ExchangeSequence::default()), 100),
+                (Arc::new(ExchangeInterRouteRandom::default()), 30),
+                (Arc::new(ExchangeIntraRouteRandom::default()), 30),
+                (Arc::new(RescheduleDeparture::default()), 20),
+            ],
+            1,
+            2,
+        ))))
+    }
 }
 
 /// Creates a mutation probability which uses `is_hit` method from passed random object.
