@@ -59,7 +59,7 @@ impl Display for DataStorage {
     }
 }
 
-pub fn create_test_network() -> Network<Data, DataStorage> {
+pub fn create_test_network(has_initial_error: bool) -> Network<Data, DataStorage> {
     Network::new(
         [
             Data::new(0.23052992, 0.95666552, 0.48200831),
@@ -67,7 +67,13 @@ pub fn create_test_network() -> Network<Data, DataStorage> {
             Data::new(0.26027299, 0.17534256, 0.19371101),
             Data::new(0.18671211, 0.16638008, 0.77362103),
         ],
-        NetworkConfig { spread_factor: 0.25, distribution_factor: 0.25, learning_rate: 0.1, rebalance_memory: 500 },
+        NetworkConfig {
+            spread_factor: 0.25,
+            distribution_factor: 0.25,
+            learning_rate: 0.1,
+            rebalance_memory: 500,
+            has_initial_error,
+        },
         Box::new(|| DataStorage::default()),
     )
 }
